@@ -22,34 +22,49 @@ class TwitterError(Exception):
     """
     pass
 
+# These actions require POST http requests instead of GET
+_POST_ACTIONS = [
+    "create", "update", "destroy", "new", "follow", "leave",
+    ]
+
 class TwitterCall(object):
-    def __init__(self, username, password, format, domain, uri=""):
+    def __init__(
+        self, username, password, format, domain, uri="", agent=None):
         self.username = username
         self.password = password
         self.format = format
-        self.uri = uri
         self.domain = domain
+        self.uri = uri
+        self.agent = agent
     def __getattr__(self, k):
         try:
             return object.__getattr__(self, k)
         except AttributeError:
             return TwitterCall(
                 self.username, self.password, self.format, self.domain,
-                self.uri + "/" + k)
+                self.uri + "/" + k, self.agent)
     def __call__(self, **kwargs):
+        uri = self.uri
         method = "GET"
-        if (self.uri.endswith('new') 
-            or self.uri.endswith('update')
-            or self.uri.endswith('create')
-            or self.uri.endswith('destroy')):
-            method = "POST"
+        for action in _POST_ACTIONS:
+            if self.uri.endswith(action):
+                method = "POST"
+                break
+            if (self.agent):
+                kwargs["source"] = self.agent
         
+        id = kwargs.pop('id', None)
+        if id:
+            uri += "/%s" %(id)
+            
         encoded_kwargs = urlencode(kwargs.items())
         argStr = ""
         if kwargs and (method == "GET"):
             argStr = "?" + encoded_kwargs
 
         headers = {}
+        if (self.agent):
+            headers["X-Twitter-Client"] = self.agent
         if (self.username):
             headers["Authorization"] = "Basic " + b64encode("%s:%s" %(
                 self.username, self.password))
@@ -60,7 +75,7 @@ class TwitterCall(object):
         c = httplib.HTTPConnection(self.domain)
         try:
             c.putrequest(method, "%s.%s%s" %(
-                self.uri, self.format, argStr))
+                uri, self.format, argStr))
             for item in headers.iteritems():
                 c.putheader(*item)
             c.endheaders()
@@ -71,8 +86,9 @@ class TwitterCall(object):
             if (r.status == 304):
                 return []
             elif (r.status != 200):
-                raise TwitterError("Twitter sent status %i: %s" %(
-                    r.status, r.read()))
+                raise TwitterError(
+                    "Twitter sent status %i for URL: %s.%s using parameters: (%s)\ndetails: %s" %(
+                        r.status, uri, self.format, encoded_kwargs, r.read()))
             if "json" == self.format:
                 return json.loads(r.read())
             else:
@@ -89,8 +105,8 @@ class Twitter(TwitterCall):
 
     The Twitter API is documented here:
 
-        http://apiwiki.twitter.com/
-        http://groups.google.com/group/twitter-development-talk/web/api-documentation
+      http://apiwiki.twitter.com/
+      http://groups.google.com/group/twitter-development-talk/web/api-documentation
     
     Examples::
     
@@ -112,13 +128,13 @@ class Twitter(TwitterCall):
 
     Searching Twitter::
         
-        twitter_search = Twitter(domain="search.twitter.com")
+      twitter_search = Twitter(domain="search.twitter.com")
 
-        # Find the latest search trends
-        twitter_search.trends()
+      # Find the latest search trends
+      twitter_search.trends()
 
-        # Search for the latest News on #gaza
-        twitter_search(q="#gaza")
+      # Search for the latest News on #gaza
+      twitter_search.search(q="#gaza")
 
     Using the data returned::
 
@@ -143,7 +159,9 @@ class Twitter(TwitterCall):
       The output will not be parsed in any way. It will be a raw string
       of XML.
     """
-    def __init__(self, email=None, password=None, format="json", domain="twitter.com"):
+    def __init__(
+        self, email=None, password=None, format="json", domain="twitter.com",
+        agent=None):
         """
         Create a new twitter API connector using the specified
         credentials (email and password). Format specifies the output
